/*
 * MPC8349 Internal Memory Map
 * Copyright (c) 2004 Freescale Semiconductor.
 * Eran Liberty (liberty@freescale.com)
 *
 * based on:
 * - MPC8260 Internal Memory Map
 *   Copyright (c) 1999 Dan Malek (dmalek@jlc.net)
 * - MPC85xx Internal Memory Map
 *   Copyright(c) 2002,2003 Motorola Inc.
 *   Xianghua Xiao (x.xiao@motorola.com)
 */
#ifndef __IMMAP_8349__
#define __IMMAP_8349__

#include <linux/types.h>
#include <asm/types.h>

typedef struct i2c
{
    u8 adr;          /**< I2C slave address              */
#define I2C_ADR	      0xFE
#define I2C_ADR_SHIFT 1
#define I2C_ADR_RES   ~(I2C_ADR)
    u8 res0[3];
    u8 fdr;          /**< I2C frequency divider register */
#define IC2_FDR       0x3F
#define IC2_FDR_SHIFT 0
#define IC2_FDR_RES   ~(IC2_FDR)
    u8 res1[3];
    u8 cr;           /**< I2C control redister           */
#define I2C_CR_MEN	  0x80
#define I2C_CR_MIEN	  0x40
#define I2C_CR_MSTA   0x20
#define I2C_CR_MTX    0x10
#define I2C_CR_TXAK   0x08
#define I2C_CR_RSTA   0x04
#define I2C_CR_BCST   0x01
    u8 res2[3];
    u8 sr;           /**< I2C status register            */
#define I2C_SR_MCF    0x80
#define I2C_SR_MAAS   0x40
#define I2C_SR_MBB    0x20
#define I2C_SR_MAL    0x10
#define I2C_SR_BCSTM  0x08
#define I2C_SR_SRW    0x04
#define I2C_SR_MIF    0x02
#define I2C_SR_RXAK   0x01
    u8 res3[3];
    u8 dr;           /**< I2C data register              */
#define I2C_DR 0xFF
#define I2C_DR_SHIFT 0
#define I2C_DR_RES ~(I2C_DR)
    u8 res4[3];
    u8 dfsrr;        /**< I2C digital filter sampling rate register */
#define I2C_DFSRR 0x3F
#define I2C_DFSRR_SHIFT 0
#define I2C_DFSRR_RES ~(I2C_DR)
    u8 res5[3];
    u8 res6[0xE8];
} i2c_t;

#if 0
#ifndef CFG_HZ
#error CFG_HZ is not defined in /include/configs/${BOARD}.h
#endif
#define I2C_TIMEOUT (CFG_HZ/4)

#ifndef CFG_IMMRBAR
#error CFG_IMMRBAR is not defined in /include/configs/${BOARD}.h
#endif

#ifndef CFG_I2C_OFFSET
#error CFG_I2C_OFFSET is not defined in /include/configs/${BOARD}.h
#endif
#endif

#if defined(CONFIG_MPC8349EMDS) || defined(CONFIG_TQM834X) || defined(CONFIG_RSSMB43) || defined(CONFIG_SIGMA)
/*
 * MPC8349 have two i2c bus
 */
extern i2c_t * mpc8349_i2c;
extern int i2c_addr_len;
#define I2C mpc8349_i2c
#else
#define I2C ((i2c_t*)(CFG_IMMRBAR + CFG_I2C_OFFSET))
#endif

#define I2C_READ  1
#define I2C_WRITE 0


/*
 * Local Access Window.
 */
typedef struct law8349 {
	u32 bar; /* LBIU local access window base address register */
/* Identifies the 20 most-significant address bits of the base of local
 * access window n. The specified base address should be aligned to the
 * window size, as defined by LBLAWARn[SIZE].
 */
#define LAWBAR_BAR         0xFFFFF000
#define LAWBAR_RES	     ~(LAWBAR_BAR)
	u32 ar; /* LBIU local access window attribute register */
} law8349_t;

/*
 * System configuration registers.
 */
typedef struct sysconf8349 {
	u32 immrbar; /* Internal memory map base address register */
	u8 res0[0x04];
	u32 altcbar; /* Alternate configuration base address register */
/* Identifies the12 most significant address bits of an alternate base
 * address used for boot sequencer configuration accesses.
 */
#define ALTCBAR_BASE_ADDR     0xFFF00000
#define ALTCBAR_RES           ~(ALTCBAR_BASE_ADDR) /* Reserved. Write has no effect, read returns 0. */
	u8 res1[0x14];
	law8349_t lblaw[4]; /* LBIU local access window */
	u8 res2[0x20];
	law8349_t pcilaw[2]; /* PCI local access window */
	u8 res3[0x10];
	law8349_t pcielaw[2];/* PCI Express local access window (8377)*/
	u8 res4[0x10];
	law8349_t ddrlaw[2]; /* DDR local access window */
	u8 res5[0x50];
	u32 sgprl; /* System General Purpose Register Low */
	u32 sgprh; /* System General Purpose Register High */
	u32 spridr; /* System Part and Revision ID Register */
	u8 res6[0x04];
	u32 spcr; /* System Priority Configuration Register */
	u32 sicrl; /* System General Purpose Register Low */
#define TSEC2_A_MASK 0x00400000
#define TSEC2_B_MASK 0x00300000
#define TSEC2_C_MASK 0x00100000
#define TSEC2_D_MASK 0x00080000
#define TSEC2_E_MASK 0x00040000
#define TSEC2_F_MASK 0x00020000
#define TSEC2_G_MASK 0x00010000
#define TSEC2_H_MASK 0x00008000
	u32 sicrh; /* System General Purpose Register High */
	u8 res7[0x04];
	u32 sidcr0;		/* System I/O Delay Configuration Register 0 */
	u32 sidcr1;		/* System I/O Delay Configuration Register 1 */
	u32 ddrcdr;		/* DDR Control Driver Register */
	u32 ddrdsr;		/* DDR Debug Status Register */
	u32 obir;		/* Output Buffer Impedance Register */
	u8 res8[0xC];
	u32 pecr1;		/* PCI Express control register 1 */
	u32 pecr2;		/* PCI Express control register 2 */
	u8 res9[0xB8];
} sysconf8349_t;

/*
 * Watch Dog Timer (WDT) Registers
 */
typedef struct wdt8349 {
	u8 res0[4];
	u32 swcrr; /* System watchdog control register */
	u32 swcnr; /* System watchdog count register */
	u8 res1[2];
	u16 swsrr; /* System watchdog service register */
	u8 res2[0xF0];
} wdt8349_t;

/*
 * RTC/PIT Module Registers
 */
typedef struct rtclk8349 {
	u32 cnr; /* control register */
#define CNR_CLEN 0x00000080 /* Clock Enable Control Bit  */
#define CNR_CLIN 0x00000040 /* Input Clock Control Bit  */
#define CNR_AIM  0x00000002 /* Alarm Interrupt Mask Bit  */
#define CNR_SIM  0x00000001 /* Second Interrupt Mask Bit  */
#define CNR_RES  ~(CNR_CLEN | CNR_CLIN | CNR_AIM | CNR_SIM)
	u32 ldr; /* load register */
	u32 psr; /* prescale register */
	u32 ctr; /* register */
	u32 evr; /* event register */
#define RTEVR_SIF  0x00000001 /* Second Interrupt Flag Bit  */
#define RTEVR_AIF  0x00000002 /* Alarm Interrupt Flag Bit  */
#define RTEVR_RES  ~(EVR_SIF | EVR_AIF)
	u32 alr; /* alarm register */
	u8 res0[0xE8];
} rtclk8349_t;

/*
 * Global timper module
 */

typedef struct gtm8349 {
	u8    cfr1; /* Timer1/2 Configuration  */
#define CFR1_PCAS 0x80 /* Pair Cascade mode  */
#define CFR1_BCM  0x40  /* Backward compatible mode  */
#define CFR1_STP2 0x20 /* Stop timer  */
#define CFR1_RST2 0x10 /* Reset timer  */
#define CFR1_GM2  0x08 /* Gate mode for pin 2  */
#define CFR1_GM1  0x04 /* Gate mode for pin 1  */
#define CFR1_STP1 0x02 /* Stop timer  */
#define CFR1_RST1 0x01 /* Reset timer  */
	u8    res0[3];
	u8    cfr2; /* Timer3/4 Configuration  */
#define CFR2_PCAS 0x80 /* Pair Cascade mode  */
#define CFR2_SCAS 0x40 /* Super Cascade mode  */
#define CFR2_STP4 0x20 /* Stop timer  */
#define CFR2_RST4 0x10 /* Reset timer  */
#define CFR2_GM4  0x08 /* Gate mode for pin 4  */
#define CFR2_GM3  0x04 /* Gate mode for pin 3  */
#define CFR2_STP3 0x02 /* Stop timer  */
#define CFR2_RST3 0x01 /* Reset timer  */
	u8    res1[10];
	u16   mdr1; /* Timer1 Mode Register  */
#define MDR_SPS  0xff00 /* Secondary Prescaler value  */
#define MDR_CE   0x00c0 /* Capture edge and enable interrupt  */
#define MDR_OM   0x0020 /* Output mode  */
#define MDR_ORI  0x0010 /* Output reference interrupt enable  */
#define MDR_FRR  0x0008 /* Free run/restart  */
#define MDR_ICLK 0x0006 /* Input clock source for the timer  */
#define MDR_GE   0x0001 /* Gate enable  */
	u16   mdr2; /* Timer2 Mode Register  */
	u16   rfr1; /* Timer1 Reference Register  */
	u16   rfr2; /* Timer2 Reference Register  */
	u16   cpr1; /* Timer1 Capture Register  */
	u16   cpr2; /* Timer2 Capture Register  */
	u16   cnr1; /* Timer1 Counter Register  */
	u16   cnr2; /* Timer2 Counter Register  */
	u16   mdr3; /* Timer3 Mode Register  */
	u16   mdr4; /* Timer4 Mode Register  */
	u16   rfr3; /* Timer3 Reference Register  */
	u16   rfr4; /* Timer4 Reference Register  */
	u16   cpr3; /* Timer3 Capture Register  */
	u16   cpr4; /* Timer4 Capture Register  */
	u16   cnr3; /* Timer3 Counter Register  */
	u16   cnr4; /* Timer4 Counter Register  */
	u16   evr1; /* Timer1 Event Register  */
	u16   evr2; /* Timer2 Event Register  */
	u16   evr3; /* Timer3 Event Register  */
	u16   evr4; /* Timer4 Event Register  */
#define GTEVR_REF 0x0002 /* Output reference event  */
#define GTEVR_CAP 0x0001 /* Counter Capture event   */
#define GTEVR_RES ~(EVR_CAP|EVR_REF)
	u16   psr1; /* Timer1 Prescaler Register  */
	u16   psr2; /* Timer2 Prescaler Register  */
	u16   psr3; /* Timer3 Prescaler Register  */
	u16   psr4; /* Timer4 Prescaler Register  */
	u8    res[0xC0];
} gtm8349_t;

/*
 * Integrated Programmable Interrupt Controller
 */
typedef struct ipic8349 {
	u32    sicfr; /*  System Global Interrupt Configuration Register (SICFR)  */
#define SICFR_HPI  0x7f000000 /*  Highest Priority Interrupt  */
#define SICFR_MPSB 0x00400000 /*  Mixed interrupts Priority Scheme for group B  */
#define SICFR_MPSA 0x00200000 /*  Mixed interrupts Priority Scheme for group A  */
#define SICFR_IPSD 0x00080000 /*  Internal interrupts Priority Scheme for group D  */
#define SICFR_IPSA 0x00010000 /*  Internal interrupts Priority Scheme for group A  */
#define SICFR_HPIT 0x00000300 /*  HPI priority position IPIC output interrupt Type  */
#define SICFR_RES ~(SICFR_HPI|SICFR_MPSB|SICFR_MPSA|SICFR_IPSD|SICFR_IPSA|SICFR_HPIT)
	u32    sivcr; /*  System Global Interrupt Vector Register (SIVCR)  */
#define SICVR_IVECX 0xfc000000 /*  Interrupt vector (for CE compatibility purpose only not used in 8349 IPIC implementation)  */
#define SICVR_IVEC  0x0000007f /*  Interrupt vector  */
#define SICVR_RES ~(SICVR_IVECX|SICVR_IVEC)
	u32    sipnr_h; /*  System Internal Interrupt Pending Register - High (SIPNR_H)  */
#define SIIH_TSEC1TX 0x80000000 /*  TSEC1 Tx interrupt  */
#define SIIH_TSEC1RX 0x40000000 /*  TSEC1 Rx interrupt  */
#define SIIH_TSEC1ER 0x20000000 /*  TSEC1 Eror interrupt  */
#define SIIH_TSEC2TX 0x10000000 /*  TSEC2 Tx interrupt  */
#define SIIH_TSEC2RX 0x08000000 /*  TSEC2 Rx interrupt  */
#define SIIH_TSEC2ER 0x04000000 /*  TSEC2 Eror interrupt  */
#define SIIH_USB2DR  0x02000000 /*  USB2 DR interrupt  */
#define SIIH_USB2MPH 0x01000000 /*  USB2 MPH interrupt  */
#define SIIH_UART1   0x00000080 /*  UART1 interrupt  */
#define SIIH_UART2   0x00000040 /*  UART2 interrupt  */
#define SIIH_SEC     0x00000020 /*  SEC interrupt  */
#define SIIH_I2C1    0x00000004 /*  I2C1 interrupt  */
#define SIIH_I2C2    0x00000002 /*  I2C1 interrupt  */
#define SIIH_SPI     0x00000001 /*  SPI interrupt  */
#define SIIH_RES	~(SIIH_TSEC1TX | SIIH_TSEC1RX | SIIH_TSEC1ER \
			| SIIH_TSEC2TX | SIIH_TSEC2RX | SIIH_TSEC2ER \
			| SIIH_USB2DR | SIIH_USB2MPH | SIIH_UART1 \
			| SIIH_UART2 | SIIH_SEC | SIIH_I2C1 \
			| SIIH_I2C2 | SIIH_SPI)
	u32    sipnr_l; /*  System Internal Interrupt Pending Register - Low (SIPNR_L)  */
#define SIIL_RTCS  0x80000000 /*  RTC SECOND interrupt  */
#define SIIL_PIT   0x40000000 /*  PIT interrupt  */
#define SIIL_PCI1  0x20000000 /*  PCI1 interrupt  */
#define SIIL_PCI2  0x10000000 /*  PCI2 interrupt  */
#define SIIL_RTCA  0x08000000 /*  RTC ALARM interrupt  */
#define SIIL_MU    0x04000000 /*  Message Unit interrupt  */
#define SIIL_SBA   0x02000000 /*  System Bus Arbiter interrupt  */
#define SIIL_DMA   0x01000000 /*  DMA interrupt  */
#define SIIL_GTM4  0x00800000 /*  GTM4 interrupt  */
#define SIIL_GTM8  0x00400000 /*  GTM8 interrupt  */
#define SIIL_GPIO1 0x00200000 /*  GPIO1 interrupt  */
#define SIIL_GPIO2 0x00100000 /*  GPIO2 interrupt  */
#define SIIL_DDR   0x00080000 /*  DDR interrupt  */
#define SIIL_LBC   0x00040000 /*  LBC interrupt  */
#define SIIL_GTM2  0x00020000 /*  GTM2 interrupt  */
#define SIIL_GTM6  0x00010000 /*  GTM6 interrupt  */
#define SIIL_PMC   0x00008000 /*  PMC interrupt  */
#define SIIL_GTM3  0x00000800 /*  GTM3 interrupt  */
#define SIIL_GTM7  0x00000400 /*  GTM7 interrupt  */
#define SIIL_GTM1  0x00000020 /*  GTM1 interrupt  */
#define SIIL_GTM5  0x00000010 /*  GTM5 interrupt  */
#define SIIL_DPTC  0x00000001 /*  DPTC interrupt (!!! Invisible for user !!!)  */
#define SIIL_RES	~(SIIL_RTCS | SIIL_PIT | SIIL_PCI1 | SIIL_PCI2 \
			| SIIL_RTCA | SIIL_MU | SIIL_SBA | SIIL_DMA \
			| SIIL_GTM4 | SIIL_GTM8 | SIIL_GPIO1 | SIIL_GPIO2 \
			| SIIL_DDR | SIIL_LBC | SIIL_GTM2 | SIIL_GTM6 \
			| SIIL_PMC |SIIL_GTM3 | SIIL_GTM7 | SIIL_GTM1 \
			| SIIL_GTM5 |SIIL_DPTC )
	u32    siprr_a; /*  System Internal Interrupt Group A Priority Register (PRR)  */
	u8     res0[8];
	u32    siprr_d; /*  System Internal Interrupt Group D Priority Register (PRR)  */
	u32    simsr_h; /*  System Internal Interrupt Mask Register - High (SIIH)  */
	u32    simsr_l; /*  System Internal Interrupt Mask Register - Low (SIIL)  */
	u8     res1[4];
	u32    sepnr;   /*  System External Interrupt Pending Register (SEI)  */
	u32    smprr_a; /*  System Mixed Interrupt Group A Priority Register (PRR)  */
	u32    smprr_b; /*  System Mixed Interrupt Group B Priority Register (PRR)  */
#define PRR_0 0xe0000000 /* Priority Register, Position 0 programming */
#define PRR_1 0x1c000000 /* Priority Register, Position 1 programming */
#define PRR_2 0x03800000 /* Priority Register, Position 2 programming */
#define PRR_3 0x00700000 /* Priority Register, Position 3 programming */
#define PRR_4 0x0000e000 /* Priority Register, Position 4 programming */
#define PRR_5 0x00001c00 /* Priority Register, Position 5 programming */
#define PRR_6 0x00000380 /* Priority Register, Position 6 programming */
#define PRR_7 0x00000070 /* Priority Register, Position 7 programming */
#define PRR_RES ~(PRR_0|PRR_1|PRR_2|PRR_3|PRR_4|PRR_5|PRR_6|PRR_7)
	u32    semsr; /*  System External Interrupt Mask Register (SEI)  */
#define SEI_IRQ0  0x80000000 /*  IRQ0 external interrupt  */
#define SEI_IRQ1  0x40000000 /*  IRQ1 external interrupt  */
#define SEI_IRQ2  0x20000000 /*  IRQ2 external interrupt  */
#define SEI_IRQ3  0x10000000 /*  IRQ3 external interrupt  */
#define SEI_IRQ4  0x08000000 /*  IRQ4 external interrupt  */
#define SEI_IRQ5  0x04000000 /*  IRQ5 external interrupt  */
#define SEI_IRQ6  0x02000000 /*  IRQ6 external interrupt  */
#define SEI_IRQ7  0x01000000 /*  IRQ7 external interrupt  */
#define SEI_SIRQ0 0x00008000 /*  SIRQ0 external interrupt  */
#define SEI_RES		~( SEI_IRQ0 | SEI_IRQ1 | SEI_IRQ2 | SEI_IRQ3 \
			| SEI_IRQ4 | SEI_IRQ5 | SEI_IRQ6 | SEI_IRQ7 \
			| SEI_SIRQ0)
	u32    secnr; /*  System External Interrupt Control Register (SECNR) */
#define SECNR_MIXB0T 0xc0000000 /*  MIXB0 priority position IPIC output interrupt type  */
#define SECNR_MIXB1T 0x30000000 /*  MIXB1 priority position IPIC output interrupt type  */
#define SECNR_MIXA0T 0x00c00000 /*  MIXA0 priority position IPIC output interrupt type  */
#define SECNR_SYSA1T 0x00300000 /*  MIXA1 priority position IPIC output interrupt type  */
#define SECNR_EDI0   0x00008000 /*  IRQ0 external interrupt edge/level detect  */
#define SECNR_EDI1   0x00004000 /*  IRQ1 external interrupt edge/level detect  */
#define SECNR_EDI2   0x00002000 /*  IRQ2 external interrupt edge/level detect  */
#define SECNR_EDI3   0x00001000 /*  IRQ3 external interrupt edge/level detect  */
#define SECNR_EDI4   0x00000800 /*  IRQ4 external interrupt edge/level detect  */
#define SECNR_EDI5   0x00000400 /*  IRQ5 external interrupt edge/level detect  */
#define SECNR_EDI6   0x00000200 /*  IRQ6 external interrupt edge/level detect  */
#define SECNR_EDI7   0x00000100 /*  IRQ7 external interrupt edge/level detect  */
#define SECNR_RES	~( SECNR_MIXB0T | SECNR_MIXB1T | SECNR_MIXA0T \
			| SECNR_SYSA1T | SECNR_EDI0 | SECNR_EDI1 \
			| SECNR_EDI2 | SECNR_EDI3 | SECNR_EDI4 \
			| SECNR_EDI5 | SECNR_EDI6 | SECNR_EDI7)
	u32   sersr; /*  System Error Status Register (SERR)  */
	u32   sermr; /*  System Error Mask Register (SERR)  */
#define SERR_IRQ0 0x80000000 /*  IRQ0 MCP request  */
#define SERR_WDT  0x40000000 /*  WDT MCP request  */
#define SERR_SBA  0x20000000 /*  SBA MCP request  */
#define SERR_DDR  0x10000000 /*  DDR MCP request  */
#define SERR_LBC  0x08000000 /*  LBC MCP request  */
#define SERR_PCI1 0x04000000 /*  PCI1 MCP request  */
#define SERR_PCI2 0x02000000 /*  PCI2 MCP request  */
#define SERR_MU   0x01000000 /*  MU MCP request  */
#define SERR_RNC  0x00010000 /*  MU MCP request (!!! Non-visible for users !!!)  */
#define SERR_RES	~( SERR_IRQ0 | SERR_WDT | SERR_SBA | SERR_DDR \
			|SERR_LBC | SERR_PCI1 | SERR_PCI2 | SERR_MU \
			|SERR_RNC )
	u32    sercr; /*  System Error Control Register  (SERCR)  */
#define SERCR_MCPR 0x00000001 /*  MCP Route  */
#define SERCR_RES ~(SERCR_MCPR)
	u8    res2[4];
	u32   sifcr_h; /*  System Internal Interrupt Force Register - High (SIIH)  */
	u32   sifcr_l; /*  System Internal Interrupt Force Register - Low (SIIL)  */
	u32   sefcr;   /*  System External Interrupt Force Register (SEI)  */
	u32   serfr;   /*  System Error Force Register (SERR)  */
	u8    res3[0xA0];
} ipic8349_t;

/*
 * System Arbiter Registers
 */
typedef struct arbiter8349 {
	u32 acr; /* Arbiter Configuration Register */
	u32 atr; /* Arbiter Timers Register */
	u8 res[4];
	u32 aer; /* Arbiter Event Register (AE)*/
	u32 aidr; /* Arbiter Interrupt Definition Register (AE) */
	u32 amr; /* Arbiter Mask Register (AE) */
	u32 aeatr; /* Arbiter Event Attributes Register */
	u32 aeadr; /* Arbiter Event Address Register */
	u32 aerr; /* Arbiter Event Response Register (AE)*/
	u8 res1[0xDC];
} arbiter8349_t;

/*
 * Reset Module
 */
typedef struct reset8349 {
	u32    rcwl; /* RCWL Register  */
	u32    rcwh; /* RCHL Register  */
	u8     res0[8];
	u32    rsr; /* Reset status Register  */
	u32    rmr; /* Reset mode Register  */
	u32    rpr; /* Reset protection Register  */
	u32    rcr; /* Reset Control Register  */
	u32    rcer; /* Reset Control Enable Register  */
	u8     res1[0xDC];
} reset8349_t;

typedef struct clk8349 {
	u32    spmr; /* system PLL mode Register  */
	u32    occr; /* output clock control Register  */
	u32    sccr; /* system clock control Register  */
	u8     res0[0xF4];
} clk8349_t;

/*
 * Power Management Control Module
 */
typedef struct pmc8349 {
	u32    pmccr; /* PMC Configuration Register  */
#define PMCCR_SLPEN 0x00000001 /* System Low Power Enable  */
#define PMCCR_DLPEN 0x00000002 /* DDR SDRAM Low Power Enable  */
#define PMCCR_RES ~(PMCCR_SLPEN | PMCCR_DLPEN)
	u32    pmcer; /* PMC Event Register  */
#define PMCER_PMCI  0x00000001 /* PMC Interrupt  */
#define PMCER_RES ~(PMCER_PMCI)
	u32    pmcmr; /* PMC Mask Register  */
#define PMCMR_PMCIE 0x0001 /* PMC Interrupt Enable  */
#define PMCMR_RES ~(PMCMR_PMCIE)
	u8 res0[0xF4];
} pmc8349_t;


/*
 * general purpose I/O module
 */
typedef struct gpio8349 {
	u32 dir; /* direction register */
	u32 odr; /* open drain register */
	u32 dat; /* data register */
	u32 ier; /* interrupt event register */
	u32 imr; /* interrupt mask register */
	u32 icr; /* external interrupt control register */
	u8 res0[0xE8];
} gpio8349_t;

/*
 * DDR Memory Controller Memory Map
 */
typedef struct ddr_cs_bnds{
	u32 csbnds;
	u8  res0[4];
} ddr_cs_bnds_t;

typedef struct ddr8349{
	ddr_cs_bnds_t csbnds[4];            /**< Chip Select x Memory Bounds */
	u8 res0[0x60];
	u32 cs_config[4];       /**< Chip Select x Configuration */
	u8 res1[0x78];
	u32 timing_cfg_1;       /**< SDRAM Timing Configuration 1 */
	u32 timing_cfg_2;       /**< SDRAM Timing Configuration 2 */
	u32 sdram_cfg;          /**< SDRAM Control Configuration */
	u8 res2[4];
	u32 sdram_mode;         /**< SDRAM Mode Configuration */
	u8 res3[8];
	u32 sdram_interval;     /**< SDRAM Interval Configuration */
	u8   res9[8];
	u32  sdram_clk_cntl;
	u8 res4[0xCCC];
	u32 data_err_inject_hi; /**< Memory Data Path Error Injection Mask High */
	u32 data_err_inject_lo; /**< Memory Data Path Error Injection Mask Low */
	u32 ecc_err_inject;     /**< Memory Data Path Error Injection Mask ECC */
	u8 res5[0x14];
	u32 capture_data_hi;    /**< Memory Data Path Read Capture High */
	u32 capture_data_lo;    /**< Memory Data Path Read Capture Low */
	u32 capture_ecc;        /**< Memory Data Path Read Capture ECC */
	u8 res6[0x14];
	u32 err_detect;         /**< Memory Error Detect */
	u32 err_disable;        /**< Memory Error Disable */
	u32 err_int_en;         /**< Memory Error Interrupt Enable */
	u32 capture_attributes; /**< Memory Error Attributes Capture */
	u32 capture_address;    /**< Memory Error Address Capture */
	u32 capture_ext_address;/**< Memory Error Extended Address Capture */
	u32 err_sbe;            /**< Memory Single-Bit ECC Error Management */
	u8 res7[0xA4];
	u32 debug_reg;
	u8 res8[0xFC];
} ddr8349_t;

/*
 * I2C1 Controller
 */


/*
 * DUART
 */
typedef struct duart8349{
	u8 urbr_ulcr_udlb; /**< combined register for URBR, UTHR and UDLB */
	u8 uier_udmb;      /**< combined register for UIER and UDMB */
	u8 uiir_ufcr_uafr; /**< combined register for UIIR, UFCR and UAFR */
	u8 ulcr;        /**< line control register */
	u8 umcr;        /**< MODEM control register */
	u8 ulsr;        /**< line status register */
	u8 umsr;        /**< MODEM status register */
	u8 uscr;        /**< scratch register */
	u8 res0[8];
	u8 udsr;        /**< DMA status register */
	u8 res1[3];
	u8 res2[0xEC];
} duart8349_t;

/*
 * Local Bus Controller Registers
 */
typedef struct lbus_bank{
	u32 br;             /**< Base Register  */
	u32 or;             /**< Base Register  */
} lbus_bank_t;

typedef struct lbus8349 {
	lbus_bank_t bank[8];
	u8 res0[0x28];
	u32 mar;                /**< UPM Address Register */
	u8 res1[0x4];
	u32 mamr;               /**< UPMA Mode Register */
	u32 mbmr;               /**< UPMB Mode Register */
	u32 mcmr;               /**< UPMC Mode Register */
	u8 res2[0x8];
	u32 mrtpr;              /**< Memory Refresh Timer Prescaler Register */
	u32 mdr;                /**< UPM Data Register */
	u8 res3[0x8];
	u32 lsdmr;              /**< SDRAM Mode Register */
	u8 res4[0x8];
	u32 lurt;               /**< UPM Refresh Timer */
	u32 lsrt;               /**< SDRAM Refresh Timer */
	u8 res5[0x8];
	u32 ltesr;              /**< Transfer Error Status Register */
	u32 ltedr;              /**< Transfer Error Disable Register */
	u32 lteir;              /**< Transfer Error Interrupt Register */
	u32 lteatr;             /**< Transfer Error Attributes Register */
	u32 ltear;              /**< Transfer Error Address Register */
	u8 res6[0xC];
	u32 lbcr;               /**< Configuration Register */
	u32 lcrr;               /**< Clock Ratio Register */
	u8 res7[0x28];
	u8 res8[0xF00];
} lbus8349_t;

/*
 * Serial Peripheral Interface
 */
typedef struct spi8349
{
	u8  res0[0x20];
	u32 mode;     /**< mode register  */
	u32 event;    /**< event register */
	u32 mask;     /**< mask register  */
	u32 com;      /**< command register */
	u32 tx;       /**< transmit register */
	u32 rx;       /**< receive register */
	u8 res1[0xC8];
} spi8349_t;


/*
 * DMA/Messaging Unit
 */
typedef struct dma8349 {
	u32 res0[0xC];	/* 0x0-0x29 reseverd */
	u32 omisr;	/* 0x30 Outbound message interrupt status register */
	u32 omimr;	/* 0x34 Outbound message interrupt mask register */
	u32 res1[0x6];	/* 0x38-0x49 reserved */

	u32 imr0;	/* 0x50 Inbound message register 0 */
	u32 imr1;	/* 0x54 Inbound message register 1 */
	u32 omr0;	/* 0x58 Outbound message register 0 */
	u32 omr1;	/* 0x5C Outbound message register 1 */

	u32 odr;	/* 0x60 Outbound doorbell register */
	u32 res2;	/* 0x64-0x67 reserved */
	u32 idr;	/* 0x68 Inbound doorbell register */
	u32 res3[0x5];	/* 0x6C-0x79 reserved */

	u32 imisr;	/* 0x80 Inbound message interrupt status register */
	u32 imimr;	/* 0x84 Inbound message interrupt mask register */
	u32 res4[0x1E];	/* 0x88-0x99 reserved */

	u32 dmamr0;	/* 0x100 DMA 0 mode register */
	u32 dmasr0;	/* 0x104 DMA 0 status register */
	u32 dmacdar0;	/* 0x108 DMA 0 current descriptor address register */
	u32 res5;	/* 0x10C reserved */
	u32 dmasar0;	/* 0x110 DMA 0 source address register */
	u32 res6;	/* 0x114 reserved */
	u32 dmadar0;	/* 0x118 DMA 0 destination address register */
	u32 res7;	/* 0x11C reserved */
	u32 dmabcr0;	/* 0x120 DMA 0 byte count register */
	u32 dmandar0;	/* 0x124 DMA 0 next descriptor address register */
	u32 res8[0x16];	/* 0x128-0x179 reserved */

	u32 dmamr1;	/* 0x180 DMA 1 mode register */
	u32 dmasr1;	/* 0x184 DMA 1 status register */
	u32 dmacdar1;	/* 0x188 DMA 1 current descriptor address register */
	u32 res9;	/* 0x18C reserved */
	u32 dmasar1;	/* 0x190 DMA 1 source address register */
	u32 res10;	/* 0x194 reserved */
	u32 dmadar1;	/* 0x198 DMA 1 destination address register */
	u32 res11;	/* 0x19C reserved */
	u32 dmabcr1;	/* 0x1A0 DMA 1 byte count register */
	u32 dmandar1;	/* 0x1A4 DMA 1 next descriptor address register */
	u32 res12[0x16];/* 0x1A8-0x199 reserved */

	u32 dmamr2;	/* 0x200 DMA 2 mode register */
	u32 dmasr2;	/* 0x204 DMA 2 status register */
	u32 dmacdar2;	/* 0x208 DMA 2 current descriptor address register */
	u32 res13;	/* 0x20C reserved */
	u32 dmasar2;	/* 0x210 DMA 2 source address register */
	u32 res14;	/* 0x214 reserved */
	u32 dmadar2;	/* 0x218 DMA 2 destination address register */
	u32 res15;	/* 0x21C reserved */
	u32 dmabcr2;	/* 0x220 DMA 2 byte count register */
	u32 dmandar2;	/* 0x224 DMA 2 next descriptor address register */
	u32 res16[0x16];/* 0x228-0x279 reserved */

	u32 dmamr3;	/* 0x280 DMA 3 mode register */
	u32 dmasr3;	/* 0x284 DMA 3 status register */
	u32 dmacdar3;	/* 0x288 DMA 3 current descriptor address register */
	u32 res17;	/* 0x28C reserved */
	u32 dmasar3;	/* 0x290 DMA 3 source address register */
	u32 res18;	/* 0x294 reserved */
	u32 dmadar3;	/* 0x298 DMA 3 destination address register */
	u32 res19;	/* 0x29C reserved */
	u32 dmabcr3;	/* 0x2A0 DMA 3 byte count register */
	u32 dmandar3;	/* 0x2A4 DMA 3 next descriptor address register */

	u32 dmagsr;	/* 0x2A8 DMA general status register */
	u32 res20[0x15];/* 0x2AC-0x2FF reserved */
} dma8349_t;

/*
 * PCI Software Configuration Registers
 */
typedef struct pciconf8349 {
	u32	config_address;
	u32 config_data;
	u32 int_ack;
	u8	res[116];
} pciconf8349_t;

/*
 * PCI Outbound Translation Register
 */
typedef struct pci_outbound_window {
	u32	potar;
	u8	res0[4];
	u32	pobar;
	u8	res1[4];
	u32	pocmr;
	u8	res2[4];
} pot8349_t;
/*
 * Sequencer
 */
typedef struct ios8349 {
	pot8349_t	pot[6];
	u8	res0[0x60];
	u32	pmcr;
	u8	res1[4];
	u32	dtcr;
	u8	res2[4];
} ios8349_t;

/*
 * PCI Controller Control and Status Registers
 */
typedef struct pcictrl8349 {
	u32	esr;
#define ESR_MERR	0x80000000
#define ESR_APAR	0x00000400
#define	ESR_PCISERR	0x00000200
#define	ESR_MPERR	0x00000100
#define	ESR_TPERR	0x00000080
#define	ESR_NORSP	0x00000040
#define	ESR_TABT	0x00000020
	u32	ecdr;
#define ECDR_APAR	0x00000400
#define	ECDR_PCISERR	0x00000200
#define	ECDR_MPERR	0x00000100
#define	ECDR_TPERR	0x00000080
#define	ECDR_NORSP	0x00000040
#define	ECDR_TABT	0x00000020
	u32 eer;
#define EER_APAR	0x00000400
#define	EER_PCISERR	0x00000200
#define	EER_MPERR	0x00000100
#define	EER_TPERR	0x00000080
#define	EER_NORSP	0x00000040
#define	EER_TABT	0x00000020
	u32	eatcr;
#define	EATCR_ERRTYPR_MASK	0x70000000
#define	EATCR_ERRTYPR_APR	0x00000000	/* address parity error */
#define	EATCR_ERRTYPR_WDPR	0x10000000	/* write data parity error */
#define	EATCR_ERRTYPR_RDPR	0x20000000	/* read data parity error */
#define	EATCR_ERRTYPR_MA	0x30000000	/* master abort */
#define	EATCR_ERRTYPR_TA	0x40000000	/* target abort */
#define	EATCR_ERRTYPR_SE	0x50000000	/* system error indication received */
#define	EATCR_ERRTYPR_PEA	0x60000000	/* parity error indication received on a read */
#define	EATCR_ERRTYPR_PEW	0x70000000	/* parity error indication received on a write */
#define EATCR_BN_MASK		0x0f000000	/* beat number */
#define	EATCR_BN_1st		0x00000000
#define	EATCR_BN_2ed		0x01000000
#define	EATCR_BN_3rd		0x02000000
#define	EATCR_BN_4th		0x03000000
#define	EATCR_BN_5th		0x0400000
#define	EATCR_BN_6th		0x05000000
#define	EATCR_BN_7th		0x06000000
#define	EATCR_BN_8th		0x07000000
#define	EATCR_BN_9th		0x08000000
#define EATCR_TS_MASK		0x00300000	/* transaction size */
#define	EATCR_TS_4		0x00000000
#define	EATCR_TS_1		0x00100000
#define	EATCR_TS_2		0x00200000
#define	EATCR_TS_3		0x00300000
#define	EATCR_ES_MASK		0x000f0000	/* error source */
#define	EATCR_ES_EM		0x00000000	/* external master */
#define	EATCR_ES_DMA		0x00050000
#define	EATCR_CMD_MASK		0x0000f000
#define	EATCR_HBE_MASK		0x00000f00	/* PCI high byte enable*/
#define	EATCR_BE_MASK		0x000000f0	/* PCI byte enable */
#define	EATCR_HPB		0x00000004	/* high parity bit */
#define	EATCR_PB		0x00000002	/* parity bit*/
#define	EATCR_VI		0x00000001	/* error information valid */
	u32	eacr;
	u32	eeacr;
	u32	edlcr;
	u32	edhcr;
	u32	gcr;
	u32	ecr;
	u32	gsr;
	u8	res0[12];
	u32	pitar2;
	u8	res1[4];
	u32	pibar2;
	u32	piebar2;
	u32	piwar2;
	u8	res2[4];
	u32	pitar1;
	u8	res3[4];
	u32	pibar1;
	u32	piebar1;
	u32	piwar1;
	u8	res4[4];
	u32	pitar0;
	u8	res5[4];
	u32	pibar0;
	u8	res6[4];
	u32	piwar0;
	u8	res7[132];
#define PITAR_TA_MASK		0x000fffff
#define PIBAR_MASK		0xffffffff
#define PIEBAR_EBA_MASK		0x000fffff
#define PIWAR_EN		0x80000000
#define PIWAR_PF		0x20000000
#define	PIWAR_RTT_MASK		0x000f0000
#define	PIWAR_RTT_NO_SNOOP	0x00040000
#define PIWAR_RTT_SNOOP		0x00050000
#define	PIWAR_WTT_MASK		0x0000f000
#define	PIWAR_WTT_NO_SNOOP	0x00004000
#define PIWAR_WTT_SNOOP		0x00005000
#define	PIWAR_IWS_MASK	0x0000003F
#define	PIWAR_IWS_4K	0x0000000B
#define	PIWAR_IWS_8K	0x0000000C
#define	PIWAR_IWS_16K	0x0000000D
#define	PIWAR_IWS_32K	0x0000000E
#define	PIWAR_IWS_64K	0x0000000F
#define	PIWAR_IWS_128K	0x00000010
#define	PIWAR_IWS_256K	0x00000011
#define	PIWAR_IWS_512K	0x00000012
#define	PIWAR_IWS_1M	0x00000013
#define	PIWAR_IWS_2M	0x00000014
#define	PIWAR_IWS_4M	0x00000015
#define	PIWAR_IWS_8M	0x00000016
#define	PIWAR_IWS_16M	0x00000017
#define	PIWAR_IWS_32M	0x00000018
#define	PIWAR_IWS_64M	0x00000019
#define	PIWAR_IWS_128M	0x0000001A
#define	PIWAR_IWS_256M	0x0000001B
#define	PIWAR_IWS_512M	0x0000001C
#define	PIWAR_IWS_1G	0x0000001D
#define	PIWAR_IWS_2G	0x0000001E
} pcictrl8349_t;

/*
 * USB
 */
typedef struct usb8349_mph {
	u8 fixme[0x1000];
} usb8349mph_t;

typedef struct usb8349_dr {
	u8 fixme[0x400];
} usb8349dr_t;

typedef struct usb8349_sys {
	u8 fixme[0xC00];
} usb8349sys_t;

/*
 * TSEC
 */
typedef struct tsec8349 {
	u8 fixme[0x1000];
} tsec8349_t;

/*
 * Security
 */
typedef struct security8349 {
	u8 fixme[0x10000];
} security8349_t;

/*
 *  PCI Express
 */
struct pex_inbound_window {
	u32 ar;
	u32 tar;
	u32 barl;
	u32 barh;
};

struct pex_outbound_window {
	u32 ar;
	u32 bar;
	u32 tarl;
	u32 tarh;
};

struct pex_csb_bridge {
	u32 pex_csb_ver;
	u32 pex_csb_cab;
	u32 pex_csb_ctrl;
	u8 res0[8];
	u32 pex_dms_dstmr;
	u8 res1[4];
	u32 pex_cbs_stat;
	u8 res2[0x20];
	u32 pex_csb_obctrl;
	u32 pex_csb_obstat;
	u8 res3[0x98];
	u32 pex_csb_ibctrl;
	u32 pex_csb_ibstat;
	u8 res4[0xb8];
	u32 pex_wdma_ctrl;
	u32 pex_wdma_addr;
	u32 pex_wdma_stat;
	u8 res5[0x94];
	u32 pex_rdma_ctrl;
	u32 pex_rdma_addr;
	u32 pex_rdma_stat;
	u8 res6[0xd4];
	u32 pex_ombcr;
	u32 pex_ombdr;
	u8 res7[0x38];
	u32 pex_imbcr;
	u32 pex_imbdr;
	u8 res8[0x38];
	u32 pex_hier;
	u32 pex_hisr;
	u32 pex_hopivr;
	u8 res9[0x14];
	u32 pex_hipivr;
	u8 res10[0x04];
	u32 pex_hwdivr;
	u8 res11[0x04];
	u32 pex_hrdivr;
	u8 res12[0x04];
	u32 pex_hmivr;
	u8 res13[0x04];
	u32 pex_cspier;
	u32 pex_cswdier;
	u32 pex_csrdier;
	u32 pex_csmier;
	u32 pex_cspisr;
	u32 pex_cswdisr;
	u32 pex_csrdisr;
	u32 pex_csmisr;
	u8 res14[0x80];
	u32 pex_pm_ctrl;
	u8 res15[0x1c];
	struct pex_outbound_window pex_outbound_win[4];
	u8 res16[0x100];
	u32 pex_epiwtar0;
	u32 pex_epiwtar1;
	u32 pex_epiwtar2;
	u32 pex_epiwtar3;
	u8 res17[0x70];
	struct pex_inbound_window pex_inbound_win[4];
};

typedef struct pex83xx {
	u8 pex_cfg_header[0x404];
	u32 pex_ltssm_stat;
	u8 res0[0x30];
	u32 pex_ack_replay_timeout;
	u8 res1[4];
	u32 pex_gclk_ratio;
	u8 res2[0xc];
	u32 pex_pm_timer;
	u32 pex_pme_timeout;
	u8 res3[4];
	u32 pex_aspm_req_timer;
	u8 res4[0x18];
	u32 pex_ssvid_update;
	u32 pex_devcap_update;
	u32 pex_linkcap_update;
	u8 res5[0x0c];
	u32 pex_slcap_update;
	u8 res6[0x1c];
	u32 pex_cfg_ready;
	u8 res7[0x24];
	u32 pex_bar_sizel;
	u8 res8[4];
	u32 pex_bar_sel;
	u8 res9[0x20];
	u32 pex_bar_pf;
	u8 res10[0x88];
	u32 pex_pme_to_ack_tor;
	u8 res11[0xc];
	u32 pex_ss_intr_mask;
	u8 res12[0x25c];
	struct pex_csb_bridge bridge;
	u8 res13[0x160];
} pex83xx_t;

/*
 * SerDes
 */
typedef struct serdes83xx {
	u32 serdes_srds1cr0;
	u32 serdes_srds1cr1;
	u32 serdes_srds1cr2;
	u32 serdes_srds1cr3;
	u32 serdes_srds1cr4;
	u8 res1[12];
	u32 srds1rstctl;
	u8 res2[220];
} serdes83xx_t;

typedef struct immap {
	sysconf8349_t sysconf; /* System configuration */
	wdt8349_t     wdt;     /* Watch Dog Timer (WDT) Registers */
	rtclk8349_t   rtc;     /* Real Time Clock Module Registers */
	rtclk8349_t   pit;     /* Periodic Interval Timer */
	gtm8349_t     gtm[2];  /* Global Timers Module */
	ipic8349_t    ipic;    /* Integrated Programmable Interrupt Controller */
	arbiter8349_t arbiter; /* System Arbiter Registers */
	reset8349_t   reset;   /* Reset Module */
	clk8349_t     clk;     /* System Clock Module */
	pmc8349_t     pmc;     /* Power Management Control Module */
	gpio8349_t    pgio[2]; /* general purpose I/O module */
	u8 res0[0x200];
	u8 DDL_DDR[0x100];
	u8 DDL_LBIU[0x100];
	u8 res1[0xE00];
	ddr8349_t     ddr;     /* DDR Memory Controller Memory */
	i2c_t     i2c[2];      /* I2C1 Controller */
	u8 res2[0x1300];
	duart8349_t   duart[2];/* DUART */
	u8 res3[0x900];
	lbus8349_t    lbus;    /* Local Bus Controller Registers */
	u8 res4[0x1000];
	spi8349_t     spi;     /* Serial Peripheral Interface */
	u8 res5[0xF00];
	dma8349_t     dma;     /* DMA */
	pciconf8349_t pci_conf[2];  /* PCI Software Configuration Registers */
	ios8349_t     ios;     /* Sequencer */
	pcictrl8349_t pci_ctrl[2];  /* PCI Controller Control and Status Registers */
	u8 res6[0x900];
	pex83xx_t     pciexp[2];/* PCI Express Controller */
	u8 res7[0x17000];
	usb8349mph_t  usb_mph;
	usb8349dr_t   usb_dr;
	usb8349sys_t  usb_sys;
	tsec8349_t    tsec[2];
	u8 res8[0xA000];
	security8349_t security;
	u8 res9[0xA3000];
	serdes83xx_t  serdes[2];/* SerDes Registers */
} immap_t;

#endif /* __IMMAP_8349__ */
