#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "rssgu.h"
#include "SgmaCommandIds.h"

#ifdef __linux__
  #include <termios.h>
  #include <unistd.h>
  #define _stricmp strcasecmp
  #define Sleep(msec) usleep((msec)*1000)

  #define __fastcall
  typedef int HANDLE;
  #define INVALID_HANDLE_VALUE -1
#else
  #pragma comment(linker, "/SUBSYSTEM:CONSOLE")

  #include "Windows.h"
  #include <conio.h>
#endif

// Controlling the serial interface for test purposes
typedef HANDLE(__fastcall *FuncSerialOpen) (const char* i_port);
typedef void(__fastcall *FuncSerialClose)(HANDLE fd_);
typedef void(__fastcall *FuncSerialSetRTS) (HANDLE fd_, int level);
typedef void(__fastcall *FuncSerialSetDTR) (HANDLE fd_, int level);
FuncSerialOpen   SerialOpen = NULL;
FuncSerialClose  SerialClose = NULL;
FuncSerialSetRTS SerialSetRTS = NULL;
FuncSerialSetDTR SerialSetDTR = NULL;
HANDLE SerialHandle = INVALID_HANDLE_VALUE;

void InitControlSerial();
void SET_RTS();
void CLR_RTS();
void SET_DTR();
void CLR_DTR();
void INIT_SERIAL();
void CLOSE_SERIAL();



#define SIGGENTYPEUNKNOWN 0
#define SIGGENTYPESGU     2

#define MAXCMDSIZE 6
#define BUFFERSIZE 1024

typedef char COMMAND [MAXCMDSIZE][100];  
#define BUFFERSIZE 1024
ViSession instrumentHandleSgs = 0;
char szResourceStringSgs [100] =  {"TCPIP::10.111.11.44::5025::SOCKET"};
ViBoolean useFastSettings = VI_TRUE;
ViBoolean enableAsynchronousFastSettings = VI_FALSE;
char g_szResourceString[1000] = {0};

void CommandLoop(ViSession instrumentHandle);
const char* DecodeVisaError (ViSession instrumentHandle, ViStatus i_stat);


// ---------------------------------------------------------------------
//
// ---------------------------------------------------------------------
void PrintCommandList()
{
    printf ("cmd:\n" \
    "\tf[?]\tFrequency\n" \
    "\tl[?]\tLevel\n" \
    "\tfl\tFrequency and Level\n" \
    "\tr[?]\tRF State\n" \
    "\tm[?]\tIQ Modulator State\n" \
    "\te\tEnumerate PCIe Bus Devices end quit program\n"
    "\ti?\tIdn Query\n" \
    "\tscpi[?]\tWrite / Query SCPI\n" \
    "\tfm[?]\tFast Mode\n" \
    "\tq\tQuit\n");
}

// ---------------------------------------------------------------------
//
// ---------------------------------------------------------------------
#ifndef __linux__

// Start SGMANDISPROT protocol driver for fast socket channel
BOOL StartStopNdis(BOOL bStart)
{
  LPCWSTR szServiceName = L"SGMANDISPROT";
  SC_HANDLE serviceDbHandle = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
  SC_HANDLE serviceHandle = serviceDbHandle ? OpenService(serviceDbHandle, szServiceName, SC_MANAGER_ALL_ACCESS) : NULL;

  SERVICE_STATUS_PROCESS status;
  DWORD bytesNeeded;
  LPWSTR szOperation = bStart ? L"start" : L"stop";
  BOOL bSuccess = TRUE;

  if (serviceHandle)
  {
    QueryServiceStatusEx(serviceHandle, SC_STATUS_PROCESS_INFO, (LPBYTE)&status, sizeof(SERVICE_STATUS_PROCESS), &bytesNeeded);

    if (status.dwCurrentState == SERVICE_RUNNING && !bStart)
      bSuccess = ControlService(serviceHandle, SERVICE_CONTROL_STOP, (LPSERVICE_STATUS)&status);
    else if (status.dwCurrentState != SERVICE_RUNNING && bStart)
      bSuccess = StartService(serviceHandle, 0, NULL);
  }
  else
  {
    bSuccess = FALSE;
  }

  if (!bSuccess)
  {
    DWORD dwLastError = GetLastError();
    WCHAR   lpBuffer[256];
    FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
      NULL,
      dwLastError,
      MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
      lpBuffer,
      255,
      NULL);
    wprintf(L"%s failed to %s: %s\n", szServiceName, szOperation, lpBuffer);
  }

  CloseServiceHandle(serviceHandle);
  CloseServiceHandle(serviceDbHandle);
  return bSuccess;
}

BOOL StartNdis()
{
  return StartStopNdis(TRUE);
}
#else
int _getch(void)
{
  struct termios oldattr, newattr;
  int ch;
  tcgetattr( STDIN_FILENO, &oldattr );
  newattr = oldattr;
  newattr.c_lflag &= ~( ICANON | ECHO );
  tcsetattr( STDIN_FILENO, TCSANOW, &newattr );
  ch = getchar();
  tcsetattr( STDIN_FILENO, TCSANOW, &oldattr );
  return ch;
}

#endif



// ---------------------------------------------------------------------
//
// ---------------------------------------------------------------------

ViStatus InterpretCommand (ViSession instrumentHandle, COMMAND szCommand)
{
  ViStatus status = VI_SUCCESS;
  double dValue = 0;
  ViBoolean bState = 0;
  char buffer [BUFFERSIZE] = {0};
  ViInt32 errorCode = 0; 

  if (!strcmp(szCommand[0], "?"))
  {
    PrintCommandList();
  }
  // Set / get frequency ------------------------- 
  else if (!strcmp(szCommand[0], "f"))
  {

    size_t len = strlen(szCommand[1]);
    double value = atof (szCommand[1]);
    if (len > 3 || *szCommand[2])
    {
      if (!_stricmp(szCommand[1]+len-3, "mhz") || !_stricmp(szCommand[2], "mhz"))
        value *= 1e6;
      if (!_stricmp(szCommand[1]+len-3, "ghz") || !_stricmp(szCommand[2], "ghz"))
        value *= 1e9;
    }
    SET_RTS();
    status = rssgu_SetFrequency(instrumentHandle, value);
    rssgu_ApplySettings(instrumentHandle);
    CLR_RTS();
  }

  else if (!strcmp(szCommand[0], "f?"))
  {
    status = rssgu_GetFrequency(instrumentHandle, &dValue);

    if (status == VI_SUCCESS)
    {
      if (dValue > 1e9)
        printf ("Freq = %f GHz\n", dValue/1e9 );
      else if (dValue > 1e6)
        printf ("Freq = %f MHz\n", dValue/1e6 );
      else 
        printf ("Freq = %f Hz\n", dValue);
    }
  }

  // Set / get level ------------------------- 
  else if (!strcmp(szCommand[0], "l"))
  {
    SET_RTS();
    status = rssgu_SetLevel(instrumentHandle, atof (szCommand[1]));
    rssgu_ApplySettings(instrumentHandle);
    CLR_RTS();
  }
  else if (!strcmp(szCommand[0], "l?"))
  {
    status = rssgu_GetLevel(instrumentHandle, &dValue);

    if (status == VI_SUCCESS)
      printf ("Level = %f dBm\n", dValue);
  }
  else if (!strcmp(szCommand[0], "lf?"))
  {
    status = rssgu_GetLoscFrequency(instrumentHandle, &dValue);
    if (status == VI_SUCCESS)
    {
      if (dValue > 1e9)
        printf ("Local freq = %f GHz\n", dValue/1e9 );
      else if (dValue > 1e6)
        printf ("local freq = %f MHz\n", dValue/1e6 );
      else 
        printf ("local freq = %f Hz\n", dValue);
    }
  }
  else if (!strcmp(szCommand[0], "ll?"))
  {
    status = rssgu_GetLoscLevel(instrumentHandle, &dValue);
    if (status == VI_SUCCESS)
    {
      printf ("Local level = %f dBm\n", dValue);
    }
  }

  // Set level and frequency ------------------------ 
  else if (!strcmp(szCommand[0], "fl"))
  {
    size_t len = 0;
    double value = 0;

    status = rssgu_SetTrigConnnectorOpMode(instrumentHandle, RSSGU_TRG_CON_OP_MODE_MEASLATENCY );
    rssgu_delay (instrumentHandle, 0.1);

    len = strlen(szCommand[1]);
    value = atof (szCommand[1]);
    if (len > 3)
    {
      if (!_stricmp(szCommand[1]+len-3, "mhz"))
        value *= 1e6;
      if (!_stricmp(szCommand[1]+len-3, "ghz"))
        value *= 1e9;
    }

    status |= rssgu_SetFrequencyAndLevel(instrumentHandle, value, atof(szCommand[2]) );
    rssgu_ApplySettings(instrumentHandle);
  }

  // Set / get RF state ------------------------- 
  else if (!strcmp(szCommand[0], "r"))
  {
    bState = !_stricmp (szCommand[1], "ON") || !_stricmp (szCommand[1],"1"); 
    status = rssgu_SetRFState(instrumentHandle, bState);
    rssgu_ApplySettings(instrumentHandle);
  }
  else if (!strcmp(szCommand[0], "r?"))
  {
    status = rssgu_GetRFState(instrumentHandle, &bState);

    if (status == VI_SUCCESS)
      printf ("RF State = %s\n", bState ? "ON" : "OFF");
  }

  // Set / get IQ modulator state ------------------------- 
  else if (!strcmp(szCommand[0], "m"))
  {
    bState = !_stricmp (szCommand[1], "ON") || !_stricmp (szCommand[1],"1"); 
    status = rssgu_SetIQModulatorState(instrumentHandle, bState);
    rssgu_ApplySettings(instrumentHandle);
  }
  else if (!strcmp(szCommand[0], "m?"))
  {
    status = rssgu_GetIQModulatorState(instrumentHandle, &bState);

    if (status == VI_SUCCESS)
      printf ("IQ Modulator State = %s\n", bState ? "ON" : "OFF");
  }

  else if (!strcmp(szCommand[0], "i?"))
  {
    status = rssgu_IdentificationQuery (instrumentHandle, buffer);

    if (status == VI_SUCCESS)
      printf ("%s\n",buffer);
  }
  // Test ------------------------- 
  else if (!strcmp(szCommand[0], "test"))
  {
    int i = 0;  
    for (i = 0; i < 1000; i++)
    {
      bState = (i & 1);
      SET_RTS();
        status = rssgu_SetRFState(instrumentHandle, bState);
      rssgu_ApplySettings(instrumentHandle);
      CLR_RTS();
      Sleep (10);
    }
  }



  // Set / get fast mode ------------------------- 
  else if (!strcmp(szCommand[0], "fm"))
  {
    useFastSettings = !_stricmp (szCommand[1], "ON") || !_stricmp (szCommand[1],"1"); 

    if (useFastSettings)
    {
      if (!_stricmp (szCommand[2], "ON") || !_stricmp (szCommand[2],"1"))
        enableAsynchronousFastSettings =  VI_TRUE;

      else if (!_stricmp (szCommand[2], "OFF") || !_stricmp (szCommand[2],"0"))
        enableAsynchronousFastSettings =  VI_FALSE;
    }

    status = rssgu_UseFastSettings (instrumentHandle, useFastSettings, enableAsynchronousFastSettings);

  }
  else if (!strcmp(szCommand[0], "fm?"))
  {
    if (status == VI_SUCCESS)
    {
      status = rssgu_GetFastSettings (instrumentHandle, &useFastSettings, &enableAsynchronousFastSettings);

      printf ("Fast Mode = %s\n", useFastSettings ? "ON" : "OFF");
      if (useFastSettings)
        printf ("Async Settings = %s\n", enableAsynchronousFastSettings ? "ON" : "OFF");
    }

  }



  else if (!strcmp(szCommand[0], "e"))
  {
    ViInt32 SerialNumbers[100];
    ViInt32 nSerialNumberCount = 100;

    status = rssgu_enumerate_pcie_devices(0x162f,0x132f, SerialNumbers, &nSerialNumberCount);

    if (status == VI_SUCCESS)
    {
      printf ("Found %d SGU device(s) on the PCIe bus:\n", (int)nSerialNumberCount);
      if (nSerialNumberCount)
      {
        int i = 0;
        for (i = 0; i < nSerialNumberCount; i++)
          printf ("%d\n", (int)SerialNumbers[i]); 
        printf ("\n");
      }
    }
    exit(0);
  }
  else if (!strcmp(szCommand[0], "scpi"))
  {
    char szArg[1000] = {0};
    int i = 1;
    while (*szCommand[i])
    {
      strcat (szArg, szCommand[i]);
      strcat (szArg, " ");
      i++;
    }

    status = rssgu_writeInstrData (instrumentHandle, szArg);
  }
  else if (!strcmp(szCommand[0], "scpi?"))
  {
    char szArg[1000] = {0};
    int i = 1;
    ViInt32 nBytesRead = 0;
    while (*szCommand[i])
    {
      strcat (szArg, szCommand[i]);
      strcat (szArg, " ");
      i++;
    }
    szArg[i] = 0;
    status = rssgu_writeInstrData (instrumentHandle, szArg);
    status = rssgu_readInstrData (instrumentHandle, 1000, szArg, &nBytesRead);
    szArg[nBytesRead] = 0;

    printf ("%s", szArg);    
  }

  else if (!strcmp(szCommand[0], "clr"))
  {
    status = rssgu_Clear(instrumentHandle);
  }  


  else
  {
    printf ("Unknown command\n");
  }

  rssgu_error_query (instrumentHandle, &errorCode, buffer);
  if (errorCode)
    printf ("%s\n", buffer);

  return status;
}

// ---------------------------------------------------------------------
//
// ---------------------------------------------------------------------

void WaitForEnterPressed()
{
  printf ("Press Enter ...");
  getchar();
}


// ---------------------------------------------------------------------
//
// ---------------------------------------------------------------------
int main(int argc, char* argv[])
{
  ViBoolean idQuery = VI_FALSE;
  ViBoolean resetDevice = VI_FALSE;
  ViSession instrumentHandle = 0;
  char buffer [BUFFERSIZE] = {0};
  ViChar instrumentIdentification[1024];
  ViStatus status = 0;
  ViInt32 errorCode = 0; 
  int SiggenType = SIGGENTYPEUNKNOWN;
  ViBoolean WaitforKey = 0;

 
  if ( argc < 2 || (argc == 2 && !strcmp(argv[1], "?")))
  {
    printf ("Usage: SguDriverDemo resourcestring [cmd arg]\n\n" \
    "resourcestring:\n" \
    "\tTCPIP::ipaddress::[hislip0::]INSTR\n" \
    "\tTCPIP::ipaddress::5025::SOCKET\n" \
    "\tUSB::0x0aad::0x00CE::serialno::INSTR\n" \
    "\tPCIE::0x0162f::0x132f::serialnoSGU::INSTR\n" \
    "\ttcpip\n" \
    "\tpcie\n" \
    );
    
    WaitForEnterPressed();
    return 1;  
  }

  InitControlSerial();
  INIT_SERIAL();
 
  strcpy (buffer, argv[1]);
  strcpy(g_szResourceString, argv[1]); 

  if ( argc == 2 && !strcmp(argv[1], "pcie"))
  {
    ViInt32 SerialNumbers[100] = {0};
    ViInt32 nSerialNumberCount = 100;

    status = rssgu_enumerate_pcie_devices(0x162f,0x132f, SerialNumbers, &nSerialNumberCount);

    if (status >= VI_SUCCESS)
    {
      int nSelection = -1;
      int i = 0;
      printf ("Found %d SGU device(s) on the PCIe bus:\n", (int)nSerialNumberCount);
      if (nSerialNumberCount == 0)
        return 0;

      for (i = 0; i < nSerialNumberCount; i++)
        printf ("%d\n", (int)SerialNumbers[i]); 
      printf ("\n");

      if (nSerialNumberCount == 1)
      {
        nSelection = 0;
      }
      else 
      {
        int c = 0;
        printf ("Select serial number (1...%d or hit ESC) ", (int)nSerialNumberCount);
        do 
        {
          c = _getch();
          if ( c-'0' >= 1 && c-'0' <= (int)nSerialNumberCount)
          {
            nSelection = c-'0' - 1;
            printf ("%c\n", c);
            break;
          }  
        } while (c != 27); // ESC
      }
      if (nSelection < 0)
        return 0;

      sprintf (buffer,"PCIe::0x162f::0x132f::%d::INSTR", (int)SerialNumbers[nSelection]);
    }
  }
  else if ( argc == 2 && !_stricmp(argv[1], "tcpip"))
  {
    int i; 
    ViInt32 infoCount = 0;
    typedef char INFOTEXT[64]; // reserve 64 chars for IP address, type , serial no

#ifndef __linux__
    if (!StartNdis())
      useFastSettings = FALSE;
#endif
    // get number of devices first
    rssgu_enumerate_socket_devices(NULL, &infoCount);
    if (infoCount > 0)
    {
      // allocate enough space for device info texts
      INFOTEXT** infoList = calloc (infoCount, sizeof (INFOTEXT*));
      for (i = 0; i < infoCount; i++)
        infoList[i] = calloc (1, sizeof (INFOTEXT));

      // get device info 
      status = rssgu_enumerate_socket_devices((ViPString *)infoList, &infoCount);
      if (status >= VI_SUCCESS)
      {
        int nSelection = -1;
        char* p=0; 
        printf ("Found %d device(s):\n", (int)infoCount);
        if (infoCount == 0)
          return 0;

        for (i = 0; i < infoCount; i++)
          printf ("%s\n", (char*)infoList[i]); 
        printf ("\n");

        if (infoCount == 1)
        {
          nSelection = 0;
        }
        else 
        {
          int c = 0;
          printf ("Select number (1...%d or hit ESC) ", (int)infoCount);
          do 
          {

            c = _getch();
            if ( c-'0' >= 1 && c-'0' <= (int)infoCount)
            {
              nSelection = c-'0' - 1;
              printf ("%c\n", c);
              break;
            }  
          } while (c != 27); // ESC
        }
        if (nSelection < 0)
        {
          printf ("\n");
          return 0;
        }

        p = strrchr((const char*)infoList[nSelection],(int)',');
        if (p)
        {
          printf ("Connecting to %s\n", p+1);
          sprintf (buffer,"TCPIP::%s::5025::SOCKET", p+1);
        }
        for (i = 0; i < infoCount; i++)
          free (infoList[i]);
        free (infoList);  
      }
    }
    else
    {
      printf ("No devices found\n");
      return;
    }
  }

  // dummy do-while provides structured exit point in case of errors
  do 
  {
    unsigned int i = 0;

#ifndef __linux__
    for (i = 0; buffer[i]; i++)
      buffer[i] = (char)tolower(buffer[i]);

    if (strstr(buffer,"::socket"))  
    {
      if (!StartNdis())
        useFastSettings = FALSE;
    }
#endif
 
    
    status = rssgu_init (buffer, idQuery, resetDevice, &instrumentHandle);
    if (status)
    {
      rssgu_error_message (instrumentHandle, status, buffer);

      printf ("open FAILED status = 0x%lx (%s)\n", status, buffer);
      WaitforKey = 1;
      break;
    }

      
    status = rssgu_IdentificationQuery (instrumentHandle, instrumentIdentification);
    if (status)
    {
      rssgu_error_message (instrumentHandle, status, buffer);
    	printf ("rssgu_IdentificationQuery failed %s\n", buffer);
      WaitforKey = 1; 
    	break;
    }
    else
    {
      if (strstr(instrumentIdentification, "SGU"))
      {
        SiggenType = SIGGENTYPESGU;
      }
      else
      {
        SiggenType = SIGGENTYPEUNKNOWN;
        printf ("Unknown Device Type\n");
      }
    }
    
    rssgu_SYSTemErrorQueueAll (instrumentHandle,BUFFERSIZE, buffer);

    status = rssgu_UseFastSettings (instrumentHandle, useFastSettings, enableAsynchronousFastSettings);
    if (status)  
    {
      WaitforKey = 1; 
      break;
    }
    status = rssgu_error_query (instrumentHandle, &errorCode, buffer);
    if (errorCode)
      printf ("\n%s", buffer);
    if (status)   
    {
      WaitforKey = 1; 
      break;
    }

    CommandLoop(instrumentHandle);
    
  } while (0);

  if (status)
  {
    rssgu_error_message (instrumentHandle, status, buffer);
    printf ("VisaError: %s\n", buffer);
  }

  status = rssgu_SYSTemErrorQueueAll (instrumentHandle,BUFFERSIZE, buffer);
  if (status == 0 && buffer[0] != '0')  
    printf ("\nError queue = %s\n", buffer);
      
  
  rssgu_close(instrumentHandle);

  printf ("\n");

#ifndef __linux__
  if (argc == 2 || WaitforKey)
    WaitForEnterPressed();
#endif

  return 0;
}



// ---------------------------------------------------------------------
//
// ---------------------------------------------------------------------
// Enter commands on the command line
void CommandLoop(ViSession instrumentHandle)
{
  char buffer[BUFFERSIZE];
  COMMAND szCommand;
  int i = 0;
  int c = 0;
  ViStatus status = VI_SUCCESS;
  
  sprintf (szCommand[0], "?");  

  printf ("SguDriverDemo %s\n", g_szResourceString);
  printf ("-------------\n");

//  if (!strncmp (buffer, "PCI", 3))
  {
    printf ("Instrument driver version "); 
    printf ("SGU: %d",SguDriverVersion);
    printf ("\n\n");
  }

  do 
  {
    status = VI_SUCCESS;

    if (!strcmp(szCommand[0], "q"))
    {
      break;
    }
    else
    {
      status = InterpretCommand(instrumentHandle, szCommand);
      if (status)
        printf ("%s (0x%x)\n", DecodeVisaError (instrumentHandle,status), (unsigned int)status);
    }


    c = 0;
    for (i = 0; i < MAXCMDSIZE; i++)
      szCommand[i][0] = 0;

    printf ("Enter command >");
    for (i = 0; i < 1000-1 && c != '\n'; ) 
    {
      c = getchar();
      if (c != '\n')
      {
        buffer[i++] = (char)c;
      }
      buffer[i] = 0;
    }
    sscanf (buffer,"%s%s%s",(char*)(szCommand+0), (char*)(szCommand+1), (char*)(szCommand+2));
    
  } while (1);
  
  CLOSE_SERIAL();

}

// ---------------------------------------------------------------------
//
// ---------------------------------------------------------------------

const char* DecodeVisaError (ViSession instrumentHandle, ViStatus i_stat)
{
  static char buffer[1000];
  ViStatus status = VI_SUCCESS;
  *buffer = 0;

  if (i_stat == VI_SUCCESS)
    return "VI_SUCCESS";

  status = rssgu_error_message (instrumentHandle, i_stat, buffer);
  if (status != VI_SUCCESS)
  {
    sprintf (buffer, "Unknown Visa Error 0x%x", (unsigned int)i_stat);
  }
  return buffer;
}

// ---------------------------------------------------------------------
// Functions for controlling the pins of a serial interface
// using an external DLL
// ---------------------------------------------------------------------

void InitControlSerial()
{
#ifndef __linux__

  void* handle = LoadLibraryA("ControlSerial.dll");
  if (handle)
  {
    SerialOpen = (FuncSerialOpen)GetProcAddress(handle, "SerialOpen");
    SerialClose = (FuncSerialClose)GetProcAddress(handle, "SerialClose");
    SerialSetRTS = (FuncSerialSetRTS)GetProcAddress(handle, "SerialSetRTS");
    SerialSetDTR = (FuncSerialSetDTR)GetProcAddress(handle, "SerialSetDTR");
  }
#endif
}

void SET_RTS()
{
  if (SerialSetRTS)
    SerialSetRTS(SerialHandle, 1);
}

void CLR_RTS()
{
  if (SerialSetRTS)
    SerialSetRTS(SerialHandle, 0);
}

void SET_DTR()
{
  if (SerialSetDTR)
    SerialSetDTR(SerialHandle, 1);
}

void CLR_DTR()
{
  if (SerialSetDTR)
    SerialSetDTR(SerialHandle, 0);
}

void INIT_SERIAL()
{
  if (SerialOpen)
    SerialHandle = SerialOpen("COM1");
}

void CLOSE_SERIAL()
{
  if (SerialClose)
    SerialClose(SerialHandle);
}

